﻿using System;
using System.Collections.Generic;

namespace SwyishChess.Domain.Piece
{
    public class Rook : ChessPiece, IVerticallyMovable, IHorizontallyMovable
    {
        /* MAXIMUM MOVABLE RANGE NOT INCLUDING THE CURRENT LOCATION OF THE CHESS PIECE */
        public const int MAXIMUM_MOVABLE_RANGE = 7;

        public Rook(int owningPlayerId)
        {
            OwningPlayerId = owningPlayerId;
        }

        /* FINDS AND RETURNS A LIST OF COORDINATES WHICH THIS CHESS PIECE CAN POSSIBLY MOVE VERTICALLY. */
        public int[,] GetPossibleVerticalMoves()
        {
            /* SOME OF THE POSSIBLE MOVES, DEENDING ONT HE CURRENT LOCATION OF THE CHESS PIECE,
             * COULD BE MOVING THE CHESS COULD 8 BLOCKS TO THE LEFT (-8) OR 
             * MOVING THE CHESS COULD 8 TO THE RIGHT (+8) */
            int possibleRows = MAXIMUM_MOVABLE_RANGE * -1;
            int[,] possibleVerticalMoves = new int[MAXIMUM_MOVABLE_RANGE * 2, 2];

            for (int row = 0; row < (MAXIMUM_MOVABLE_RANGE * 2); row++)
            {
                /* EXCLUDES THE CHESS PIECES CURRENT LOCATION WHICH WOULD BE [0, 0] */

                possibleVerticalMoves[row, 0] = possibleRows;
                possibleVerticalMoves[row, 1] = 0;

                possibleRows++;
                if (possibleRows == 0)
                    possibleRows++;
            }

            return possibleVerticalMoves;
        }

        /* FINDS AND RETURNS A LIST OF COORDINATES WHICH THIS CHESS PIECES CAN POSSIBLE MOVE HORIZONTALLY. */
        public int[,] GetPossibleHorizontalMoves()
        {
            int possibleColumns = MAXIMUM_MOVABLE_RANGE * -1;
            int[,] possibleHorizontalMoves = new int[MAXIMUM_MOVABLE_RANGE * 2, 2];

            for (int row = 0; row < (MAXIMUM_MOVABLE_RANGE * 2); row++)
            {
                possibleHorizontalMoves[row, 0] = 0;
                possibleHorizontalMoves[row, 1] = possibleColumns;

                possibleColumns++;
                if (possibleColumns == 0)
                    possibleColumns++;
            }

            return possibleHorizontalMoves;
        }

        /* FINDS AND RETURNS ALL THE MOVES POSSIBLE BY THIS CHESS PIECE */
        public override List<int[,]> AllPossibleMoves
        {
            get
            {
                List<int[,]> possibleMoves = new List<int[,]>();
                possibleMoves.Add(GetPossibleHorizontalMoves());
                possibleMoves.Add(GetPossibleVerticalMoves());
                return possibleMoves;
            }
        }

        public override string ToString()
        {
            return "Rook";
        }
    }
}
